const path = require('path')
const utils = require('./utils')
const config = require(`${process.cwd()}/config`)
const vueLoaderConfig = require('./vue-loader.conf')
const MpvuePlugin = require('webpack-mpvue-asset-plugin')
const glob = require('glob')
const CopyWebpackPlugin = require('copy-webpack-plugin')
const CleanWebpackPlugin = require('clean-webpack-plugin')
const relative = require('relative')
const HappyPack = require('happypack')
const os = require('os')
const happyThreadPool = HappyPack.ThreadPool({ size: os.cpus().length })

function resolve (dir) {
  return path.join(process.cwd(), dir)
}

function getEntry (rootSrc) {
  const map = {}
  glob.sync(rootSrc + '/pages/**/main.js')
    .forEach(file => {
      const key = relative(rootSrc, file).replace('.js', '')
      map[key] = file
    })

  return map
}

const appEntry = { app: resolve('./src/main.js') }
const pagesEntry = getEntry(resolve('./src'), 'pages/**/main.js')
const entry = Object.assign({}, appEntry, pagesEntry)

// 设置app.json下的pages自动生成
// 会存在覆盖加载顺序，故注释
// let appJson = require(`${process.cwd()}/src/app.json`)
// appJson.pages = [...Object.keys(pagesEntry)]
// fs.writeFileSync(`${process.cwd()}/src/app.json`, JSON.stringify(appJson, null, 2))

module.exports = {
  // 如果要自定义生成的 dist 目录里面的文件路径，
  // 可以将 entry 写成 {'toPath': 'fromPath'} 的形式，
  // toPath 为相对于 dist 的路径, 例：index/demo，则生成的文件地址为 dist/index/demo.js
  entry,
  target: require('mpvue-webpack-target'),
  output: {
    path: config.build.assetsRoot,
    filename: '[name].js',
    publicPath: process.env.NODE_ENV === 'production'
      ? config.build.assetsPublicPath
      : config.dev.assetsPublicPath
  },
  resolve: {
    extensions: ['.js', '.vue', '.json'],
    modules: [
      resolve('src'),
      resolve('node_modules')
    ],
    alias: {
      'vue': 'mpvue',
      '@': resolve('src')
    },
    symlinks: false,
    aliasFields: ['mpvue', 'weapp', 'browser'],
    mainFields: ['browser', 'module', 'main']
  },
  module: {
    rules: [
      {
        test: /\.(js|vue)$/,
        loader: 'eslint-loader',
        enforce: 'pre',
        include: [resolve('src'), resolve('test')],
        options: {
          formatter: require('eslint-friendly-formatter')
        }
      },
      {
        test: /\.vue$/,
        loader: 'mpvue-loader',
        options: vueLoaderConfig,
        include: [resolve('src'), resolve('node_modules/vue-easytable/libs')],
        exclude: /node_modules\/(?!(autotrack|dom-utils))|vendor\.dll\.js/
      },
      {
        test: /\.js$/,
        include: [resolve('src'), resolve('test')],
        exclude: /node_modules/,
        use: [
          'babel-loader',
          'happypack/loader?id=happyjs',
          {
            loader: 'mpvue-loader',
            options: {
              checkMPEntry: true
            }
          }
        ]
      },
      {
        test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10000,
          name: utils.assetsPath('img/[name].[ext]')
        }
      },
      {
        test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10000,
          name: utils.assetsPath('media/[name].[ext]')
        }
      },
      {
        test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10000,
          name: utils.assetsPath('fonts/[name].[ext]')
        }
      }
    ]
  },
  plugins: [
    new CleanWebpackPlugin([`${process.cwd()}/dist`], {
      root: process.cwd()
    }),
    new MpvuePlugin(),
    new CopyWebpackPlugin([{
      from: '**/*.json',
      to: ''
    }], {
      context: 'src/'
    }),
    new CopyWebpackPlugin([
      {
        from: path.resolve(__dirname, `${process.cwd()}/static`),
        to: path.resolve(__dirname, `${process.cwd()}/dist/static`),
        ignore: ['.*']
      }
    ]),
    new HappyPack({
      //用id来标识 happypack处理那里类文件
      id: 'happyjs',
      //如何处理  用法和loader 的配置一样
      loaders: [{
        loader: 'babel-loader?cacheDirectory=true'
      }],
      //共享进程池
      threadPool: happyThreadPool,
      //允许 HappyPack 输出日志
      verbose: true
    })
    // // 添加DllReferencePlugin插件
    // new webpack.DllReferencePlugin({
    //   context: __dirname,
    //   manifest: require(`${process.cwd()}/static/vendor/vendor-manifest.json`)
    // })
  ]
}
